<?php
class Magento1_FeedbackCompany_Model_Options extends Mage_Core_Model_Abstract
{
    const API_ENDPOINT_URI      = 'https://www.feedbackcompany.com/api/v2/';
    const API_ACCESS_TOKEN_PATH = 'oauth2/token';
    const API_WIDGETS_PATH      = 'widgets';
    const API_ORDERS_PATH       = 'orders';

    protected $_httpConfig = array(
        'timeout'        => 3,
        'maxredirects'   => 0,
        'returntransfer' => true
    );

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init('magento1_feedbackcompany/options');
        parent::_construct();
    }

    /**
     * Load feedback company configuration item
     *
     * @return $this
     */
    public function loadConfig()
    {
        $resource = $this->_getResource();
        $resource->loadConfig($this);

        return $this;
    }

    /**
     * Get new access token
     *
     * @return string
     */
    public function getNewAccessToken()
    {
        return $this->retrieveAccessToken();
    }

    /**
     * Register widgets in feedback company
     *
     * @return string
     */
    public function registerWidgets()
    {
        $errors = array();
        $params = array();
        $dbWidgetField = '';
        $widgetTypes = array('main', 'product-summary', 'product-extended');
        $apiUriPart = self::API_WIDGETS_PATH;
        $apiMethod = Zend_Http_Client::POST;

        foreach ($widgetTypes as $widgetType) {
            switch ($widgetType) {
                case 'main':
                    $dbWidgetField = 'merchant_reviews_widget_uuid';
                    $params = array(
                        'type'     => $widgetType,
                        'options' => array(
                            'size'              => $this->getData('merchant_reviews_widget_size'),
                            'amount_of_reviews' => (int) $this->getData('merchant_reviews_widget_amount_of_reviews')
                        )
                    );
                    break;
                case 'product-summary':
                    $dbWidgetField = 'product_reviews_summary_widget_uuid';
                    $params = array(
                        'type'     => $widgetType
                    );
                    break;
               case 'product-extended':
                   $dbWidgetField = 'product_reviews_extended_widget_uuid';
                   $params = array(
                       'type'     => $widgetType,
                       'options' => array(
                           'display_type' => $this->getData('product_reviews_widget_extended_display_type')
                       )
                   );
                   break;
            }

            $apiResult = $this->callApi($apiUriPart, $apiMethod, $params);
            $result = $this->registerWidget($apiResult, $dbWidgetField);
            if (is_array($result)) {
                $errors[] = $result['error'];
            }
        }

        return $errors;
    }

    /**
     * Register order items in feedback company
     *
     * @return string
     */
    public function registerOrder($order)
    {
        $products = array();
        foreach ($order->getAllVisibleItems() as $item) {
            $product = $item->getProduct();

            $data = array(
                'external_id' => $product->getSku(),
                'sku' => $item->getSku(),
                'name' => $item->getName(),
                'url' => $product->getProductUrl()
            );

            try {
                $data['image_url'] = (string) Mage::helper('catalog/image')->init($product, 'image');
            } catch (Exception $e) {
                Mage::log('Unable to get image for product with sku: ' . $product->getSku());
            }

            if ($product->getGtin()) {
                $data['gtin'] = $product->getGtin();
            }

            $products[] = $data;
        }

        //register by api
        $apiUriPart = self::API_ORDERS_PATH;
        $apiMethod = Zend_Http_Client::POST;
        $orderId = $order->getIncrementId();
        $params = array(
            'external_id' => "order-{{$orderId}}",
            'customer' => array(
                'email' => $order->getCustomerEmail(),
                'fullname' => $order->getCustomerName()
            ),
            'products' => $products,
            'invitation' => array(
                'delay' => array(
                    'unit' => $this->getData('review_invitation_delay_period'),
                    'amount' => $this->getData('review_invitation_delay_amount')
                ),
                'reminder' => array(
                    'unit' => $this->getData('review_invitation_reminder_delay_period')
                )
            )
        );

        if ((int)$this->getData('review_invitation_reminder_delay_amount') > 0) {
            $params['invitation']['reminder']['amount'] = $this->getData('review_invitation_reminder_delay_amount');
        }

        $apiResult = $this->callApi($apiUriPart, $apiMethod, $params);
        $response = $apiResult['response'];

        // log response error
        if (isset($response['error'])) {
            Mage::log($response['error']);
        }
    }

    /**
     * Retrieve new access token
     *
     * @return bool|array
     */
    private function retrieveAccessToken()
    {
        $apiUriPart = self::API_ACCESS_TOKEN_PATH;
        $params = $this->buildAccessTokenParams('authorization_code');
        $apiMethod = Zend_Http_Client::GET;
        $apiResult = $this->callApi($apiUriPart, $apiMethod, $params);
        $response = $apiResult['response'];

        $result = true;
        if ($apiResult['success'] && isset($response['error']) && !$response['error']) {
            $accessToken = isset($response['access_token']) ? $response['access_token'] : '';
            $this->setData('access_token', $accessToken);
        }

        if (!$apiResult['success'] && isset($response['error']) && $response['error']) {
            $result = array('error' => "Failed to fetch a new access token from Feedback Company API" . $response['error']);
        }

        return $result;
    }

    /**
     * Register Widget
     *
     * @param array $apiResult
     * @param string $dbWidgetField
     *
     * @return bool|array
     */
    private function registerWidget($apiResult, $dbWidgetField)
    {
        $response = $apiResult['response'];
        $result = true;

        if ($apiResult['success'] && !isset($response['error'])) {
            $widget = $response['widget']['uuid'];
            $this->setData($dbWidgetField, $widget);
        }

        if (!$apiResult['success'] && isset($response['error']) && $response['error']) {
            $result = array('error' => $response['error']);
            Mage::log($response['error']);
        }

        return $result;
    }

    /**
     * API endpoint getter
     *
     * @param string $grantType
     * @return array
     */
    private function buildAccessTokenParams($grantType)
    {
        $data = array(
            'client_id'     => $this->getData('client_id'),
            'client_secret' => $this->getData('client_secret'),
            'grant_type'    => $grantType
        );

        return $data;
    }

    /**
     * Call Feedback Company Api
     *
     * @param string $apiUriPart
     * @param string $apiMethod
     * @param array $params
     * @return array
     */
    private function callApi($apiUriPart, $apiMethod, $params)
    {
        try {
            $client = new Zend_Http_Client();
            $client->setUri(self::API_ENDPOINT_URI . $apiUriPart);
            $client->setConfig($this->_httpConfig);
            $client->setHeaders('Authorization', 'Bearer ' . $this->getData('access_token'));
            $client->setHeaders('Content-Type', 'application/json');

            switch ($apiMethod) {
                case Zend_Http_Client::GET :
                    $client->setParameterGet($params);
                    break;
                case Zend_Http_Client::POST:
                    $client->setRawData(Mage::helper('core')->jsonEncode($params));
                    $client->setMethod($apiMethod);
                    break;
            }

            $response = $client->request($apiMethod);
            $responseData = Mage::helper('core')->jsonDecode($response->getBody());
            $success = false;
            if (in_array($response->getStatus(), range(200, 299))) {
                $success = true;
            }
        } catch (Zend_Http_Client_Exception $e) {
            $success = false;
            $responseData = array('error' => true, 'description' => $e->getMessage());
        }

        return array('success' => $success, 'response' => $responseData);
    }

    /**
     * Retrieve review delay periods options
     *
     * @return array
     */
    public function getReviewDelayPeriodOptions()
    {
        $helper = Mage::helper('magento1_feedbackcompany');

        return array(
            'minutes'  => $helper->__('Minutes'),
            'hours'    => $helper->__('Hours'),
            'days'     => $helper->__('Days'),
            'weekdays' => $helper->__('Weekdays')
        );
    }

    /**
     * Retrieve merchant reviews widget size options
     *
     * @return array
     */
    public function getMerchantReviewsWidgetSizeOptions()
    {
        $helper = Mage::helper('magento1_feedbackcompany');

        return array(
            'small' => $helper->__('Small'),
            'big'   => $helper->__('Big')
        );
    }

    /**
     * Retrieve merchant reviews widget amount of reviews options
     *
     * @return array
     */
    public function getMerchantReviewsWidgetAmountOfReviewsOptions()
    {
        return array(
            '0' => '0',
            '1' => '1',
            '2' => '2'
        );
    }
}
